@@ -194,7 +194,7 @@ gst_ffmpegmux_base_init (gpointer g_class)
   AVOutputFormat *in_plugin;
   GstCaps *srccaps, *audiosinkcaps, *videosinkcaps;
   enum AVCodecID *video_ids = NULL, *audio_ids = NULL;
-  gchar *longname, *description;
+  gchar *longname, *description, *name;
   const char *replacement;
   gboolean is_formatter;
 
@@ -203,6 +203,9 @@ gst_ffmpegmux_base_init (gpointer g_class)
       GST_FFMUX_PARAMS_QDATA);
   g_assert (in_plugin != NULL);
 
+  name = g_strdup (in_plugin->name);
+  g_strdelimit (name, ".,|-<> ", '_');
+
   /* construct the element details struct */
   replacement = gst_ffmpegmux_get_replacement (in_plugin->name);
   is_formatter = gst_ffmpegmux_is_formatter (in_plugin->name);
@@ -229,19 +232,17 @@ gst_ffmpegmux_base_init (gpointer g_class)
   g_free (description);
 
   /* Try to find the caps that belongs here */
-  srccaps = gst_ffmpeg_formatid_to_caps (in_plugin->name);
+  srccaps = gst_ffmpeg_formatid_to_caps (name);
   if (!srccaps) {
-    GST_DEBUG ("Couldn't get source caps for muxer '%s', skipping format",
-        in_plugin->name);
+    GST_DEBUG ("Couldn't get source caps for muxer '%s', skipping", name);
     goto beach;
   }
 
   if (!gst_ffmpeg_formatid_get_codecids (in_plugin->name,
           &video_ids, &audio_ids, in_plugin)) {
     gst_caps_unref (srccaps);
-    GST_DEBUG
-        ("Couldn't get sink caps for muxer '%s'. Most likely because no input format mapping exists.",
-        in_plugin->name);
+    GST_DEBUG ("Couldn't get sink caps for muxer '%s'. Most likely because "
+        "no input format mapping exists.", name);
     goto beach;
   }
 
@@ -268,21 +269,26 @@ gst_ffmpegmux_base_init (gpointer g_class)
   /* pad templates */
   srctempl = gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS, srccaps);
   gst_element_class_add_pad_template (element_class, srctempl);
+  gst_caps_unref (srccaps);
 
   if (audiosinkcaps) {
     audiosinktempl = gst_pad_template_new ("audio_%u",
         GST_PAD_SINK, GST_PAD_REQUEST, audiosinkcaps);
     gst_element_class_add_pad_template (element_class, audiosinktempl);
+    gst_caps_unref (audiosinkcaps);
   }
 
   if (videosinkcaps) {
     videosinktempl = gst_pad_template_new ("video_%u",
         GST_PAD_SINK, GST_PAD_REQUEST, videosinkcaps);
     gst_element_class_add_pad_template (element_class, videosinktempl);
+    gst_caps_unref (videosinkcaps);
   }
 
 beach:
   klass->in_plugin = in_plugin;
+
+  g_free (name);
 }
 
 static void
@@ -909,7 +915,6 @@ gst_ffmpegmux_register (GstPlugin * plugin)
 
   while (in_plugin) {
     gchar *type_name;
-    gchar *p;
     GstRank rank = GST_RANK_MARGINAL;
 
     if ((!strncmp (in_plugin->name, "u16", 3)) ||
@@ -926,6 +931,7 @@ gst_ffmpegmux_register (GstPlugin * plugin)
         (!strncmp (in_plugin->name, "crc", 3)) ||
         (!strncmp (in_plugin->name, "null", 4)) ||
         (!strncmp (in_plugin->name, "gif", 3)) ||
+        (!strncmp (in_plugin->name, "fifo", 4)) ||
         (!strncmp (in_plugin->name, "frame", 5)) ||
         (!strncmp (in_plugin->name, "image", 5)) ||
         (!strncmp (in_plugin->name, "mulaw", 5)) ||
@@ -935,6 +941,7 @@ gst_ffmpegmux_register (GstPlugin * plugin)
         (!strncmp (in_plugin->name, "ass", 3)) ||
         (!strncmp (in_plugin->name, "ffmetadata", 10)) ||
         (!strncmp (in_plugin->name, "srt", 3)) ||
+        (!strncmp (in_plugin->name, "scc", 3)) ||
         !strcmp (in_plugin->name, "segment") ||
         !strcmp (in_plugin->name, "stream_segment,ssegment") ||
         !strcmp (in_plugin->name, "jacosub") ||
@@ -961,16 +968,7 @@ gst_ffmpegmux_register (GstPlugin * plugin)
 
     /* construct the type */
     type_name = g_strdup_printf ("avmux_%s", in_plugin->name);
-
-    p = type_name;
-
-    while (*p) {
-      if (*p == '.')
-        *p = '_';
-      if (*p == ',')
-        *p = '_';
-      p++;
-    }
+    g_strdelimit (type_name, ".,|-<> ", '_');
 
     type = g_type_from_name (type_name);
 
