@@ -25,6 +25,7 @@
 #include <string.h>
 
 #include <libavcodec/avcodec.h>
+#include <libavutil/stereo3d.h>
 
 #include <gst/gst.h>
 #include <gst/video/video.h>
@@ -192,6 +193,9 @@ gst_ffmpegviddec_base_init (GstFFMpegVidDecClass * klass)
   gst_element_class_add_pad_template (element_class, srctempl);
   gst_element_class_add_pad_template (element_class, sinktempl);
 
+  gst_caps_unref (sinkcaps);
+  gst_caps_unref (srccaps);
+
   klass->in_plugin = in_plugin;
 }
 
@@ -433,9 +437,14 @@ gst_ffmpegviddec_set_format (GstVideoDecoder * decoder,
     ffmpegdec->pic_height = 0;
     ffmpegdec->pic_par_n = 0;
     ffmpegdec->pic_par_d = 0;
+    ffmpegdec->pic_interlaced = 0;
+    ffmpegdec->pic_field_order = 0;
+    ffmpegdec->pic_field_order_changed = FALSE;
     ffmpegdec->ctx_ticks = 0;
     ffmpegdec->ctx_time_n = 0;
     ffmpegdec->ctx_time_d = 0;
+    ffmpegdec->cur_multiview_mode = GST_VIDEO_MULTIVIEW_MODE_NONE;
+    ffmpegdec->cur_multiview_flags = GST_VIDEO_MULTIVIEW_FLAGS_NONE;
   }
 
   gst_caps_replace (&ffmpegdec->last_caps, state->caps);
@@ -881,12 +890,24 @@ no_frame:
 static gboolean
 picture_changed (GstFFMpegVidDec * ffmpegdec, AVFrame * picture)
 {
+  gint pic_field_order = 0;
+
+  if (picture->interlaced_frame) {
+    if (picture->repeat_pict)
+      pic_field_order |= GST_VIDEO_BUFFER_FLAG_RFF;
+    if (picture->top_field_first)
+      pic_field_order |= GST_VIDEO_BUFFER_FLAG_TFF;
+  }
+
   return !(ffmpegdec->pic_width == picture->width
       && ffmpegdec->pic_height == picture->height
       && ffmpegdec->pic_pix_fmt == picture->format
       && ffmpegdec->pic_par_n == picture->sample_aspect_ratio.num
       && ffmpegdec->pic_par_d == picture->sample_aspect_ratio.den
-      && ffmpegdec->pic_interlaced == picture->interlaced_frame);
+      && ffmpegdec->pic_interlaced == picture->interlaced_frame
+      && ffmpegdec->pic_field_order == pic_field_order
+      && ffmpegdec->cur_multiview_mode == ffmpegdec->picture_multiview_mode
+      && ffmpegdec->cur_multiview_flags == ffmpegdec->picture_multiview_flags);
 }
 
 static gboolean
@@ -901,6 +922,15 @@ static gboolean
 update_video_context (GstFFMpegVidDec * ffmpegdec, AVCodecContext * context,
     AVFrame * picture)
 {
+  gint pic_field_order = 0;
+
+  if (picture->interlaced_frame) {
+    if (picture->repeat_pict)
+      pic_field_order |= GST_VIDEO_BUFFER_FLAG_RFF;
+    if (picture->top_field_first)
+      pic_field_order |= GST_VIDEO_BUFFER_FLAG_TFF;
+  }
+
   if (!picture_changed (ffmpegdec, picture)
       && !context_changed (ffmpegdec, context))
     return FALSE;
@@ -921,7 +951,23 @@ update_video_context (GstFFMpegVidDec * ffmpegdec, AVCodecContext * context,
   ffmpegdec->pic_height = picture->height;
   ffmpegdec->pic_par_n = picture->sample_aspect_ratio.num;
   ffmpegdec->pic_par_d = picture->sample_aspect_ratio.den;
+  ffmpegdec->cur_multiview_mode = ffmpegdec->picture_multiview_mode;
+  ffmpegdec->cur_multiview_flags = ffmpegdec->picture_multiview_flags;
+
+  /* Remember if we have interlaced content and the field order changed
+   * at least once. If that happens, we must be interlace-mode=mixed
+   */
+  if (ffmpegdec->pic_field_order_changed ||
+      (ffmpegdec->pic_field_order != pic_field_order &&
+          ffmpegdec->pic_interlaced))
+    ffmpegdec->pic_field_order_changed = TRUE;
+
+  ffmpegdec->pic_field_order = pic_field_order;
   ffmpegdec->pic_interlaced = picture->interlaced_frame;
+
+  if (!ffmpegdec->pic_interlaced)
+    ffmpegdec->pic_field_order_changed = FALSE;
+
   ffmpegdec->ctx_ticks = context->ticks_per_frame;
   ffmpegdec->ctx_time_n = context->time_base.num;
   ffmpegdec->ctx_time_d = context->time_base.den;
@@ -1003,6 +1049,31 @@ no_par:
   }
 }
 
+static GstVideoMultiviewMode
+stereo_av_to_gst (enum AVStereo3DType type)
+{
+  switch (type) {
+    case AV_STEREO3D_SIDEBYSIDE:
+      return GST_VIDEO_MULTIVIEW_MODE_SIDE_BY_SIDE;
+    case AV_STEREO3D_TOPBOTTOM:
+      return GST_VIDEO_MULTIVIEW_MODE_TOP_BOTTOM;
+    case AV_STEREO3D_FRAMESEQUENCE:
+      return GST_VIDEO_MULTIVIEW_MODE_FRAME_BY_FRAME;
+    case AV_STEREO3D_CHECKERBOARD:
+      return GST_VIDEO_MULTIVIEW_MODE_CHECKERBOARD;
+    case AV_STEREO3D_SIDEBYSIDE_QUINCUNX:
+      return GST_VIDEO_MULTIVIEW_MODE_SIDE_BY_SIDE_QUINCUNX;
+    case AV_STEREO3D_LINES:
+      return GST_VIDEO_MULTIVIEW_MODE_ROW_INTERLEAVED;
+    case AV_STEREO3D_COLUMNS:
+      return GST_VIDEO_MULTIVIEW_MODE_COLUMN_INTERLEAVED;
+    default:
+      break;
+  }
+
+  return GST_VIDEO_MULTIVIEW_MODE_NONE;
+}
+
 static gboolean
 gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
     AVCodecContext * context, AVFrame * picture)
@@ -1012,6 +1083,7 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
   GstVideoCodecState *output_state;
   gint fps_n, fps_d;
   GstClockTime latency;
+  GstStructure *in_s;
 
   if (!update_video_context (ffmpegdec, context, picture))
     return TRUE;
@@ -1031,26 +1103,143 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
   out_info = &ffmpegdec->output_state->info;
 
   /* set the interlaced flag */
-  if (ffmpegdec->pic_interlaced)
-    out_info->interlace_mode = GST_VIDEO_INTERLACE_MODE_MIXED;
-  else
-    out_info->interlace_mode = GST_VIDEO_INTERLACE_MODE_PROGRESSIVE;
+  in_s = gst_caps_get_structure (ffmpegdec->input_state->caps, 0);
 
-  switch (context->chroma_sample_location) {
-    case AVCHROMA_LOC_LEFT:
-      out_info->chroma_site = GST_VIDEO_CHROMA_SITE_MPEG2;
-      break;
-    case AVCHROMA_LOC_CENTER:
-      out_info->chroma_site = GST_VIDEO_CHROMA_SITE_JPEG;
-      break;
-    case AVCHROMA_LOC_TOPLEFT:
-      out_info->chroma_site = GST_VIDEO_CHROMA_SITE_DV;
-      break;
-    case AVCHROMA_LOC_TOP:
-      out_info->chroma_site = GST_VIDEO_CHROMA_SITE_V_COSITED;
-      break;
-    default:
-      break;
+  if (!gst_structure_has_field (in_s, "interlace-mode")) {
+    if (ffmpegdec->pic_interlaced) {
+      if (ffmpegdec->pic_field_order_changed ||
+          (ffmpegdec->pic_field_order & GST_VIDEO_BUFFER_FLAG_RFF)) {
+        out_info->interlace_mode = GST_VIDEO_INTERLACE_MODE_MIXED;
+      } else {
+        out_info->interlace_mode = GST_VIDEO_INTERLACE_MODE_INTERLEAVED;
+        if ((ffmpegdec->pic_field_order & GST_VIDEO_BUFFER_FLAG_TFF))
+          GST_VIDEO_INFO_FIELD_ORDER (out_info) =
+              GST_VIDEO_FIELD_ORDER_TOP_FIELD_FIRST;
+        else
+          GST_VIDEO_INFO_FIELD_ORDER (out_info) =
+              GST_VIDEO_FIELD_ORDER_BOTTOM_FIELD_FIRST;
+      }
+    } else {
+      out_info->interlace_mode = GST_VIDEO_INTERLACE_MODE_PROGRESSIVE;
+    }
+  }
+
+  if (!gst_structure_has_field (in_s, "chroma-site")) {
+    switch (context->chroma_sample_location) {
+      case AVCHROMA_LOC_LEFT:
+        out_info->chroma_site = GST_VIDEO_CHROMA_SITE_MPEG2;
+        break;
+      case AVCHROMA_LOC_CENTER:
+        out_info->chroma_site = GST_VIDEO_CHROMA_SITE_JPEG;
+        break;
+      case AVCHROMA_LOC_TOPLEFT:
+        out_info->chroma_site = GST_VIDEO_CHROMA_SITE_DV;
+        break;
+      case AVCHROMA_LOC_TOP:
+        out_info->chroma_site = GST_VIDEO_CHROMA_SITE_V_COSITED;
+        break;
+      default:
+        break;
+    }
+  }
+
+  if (!gst_structure_has_field (in_s, "colorimetry")
+      || in_info->colorimetry.primaries == GST_VIDEO_COLOR_PRIMARIES_UNKNOWN) {
+    switch (context->color_primaries) {
+      case AVCOL_PRI_BT709:
+        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT709;
+        break;
+      case AVCOL_PRI_BT470M:
+        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT470M;
+        break;
+      case AVCOL_PRI_BT470BG:
+        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT470BG;
+        break;
+      case AVCOL_PRI_SMPTE170M:
+        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_SMPTE170M;
+        break;
+      case AVCOL_PRI_SMPTE240M:
+        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_SMPTE240M;
+        break;
+      case AVCOL_PRI_FILM:
+        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_FILM;
+        break;
+      case AVCOL_PRI_BT2020:
+        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT2020;
+        break;
+      default:
+        break;
+    }
+  }
+
+  if (!gst_structure_has_field (in_s, "colorimetry")
+      || in_info->colorimetry.transfer == GST_VIDEO_TRANSFER_UNKNOWN) {
+    switch (context->color_trc) {
+      case AVCOL_TRC_BT2020_10:
+      case AVCOL_TRC_BT709:
+      case AVCOL_TRC_SMPTE170M:
+        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_BT709;
+        break;
+      case AVCOL_TRC_GAMMA22:
+        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_GAMMA22;
+        break;
+      case AVCOL_TRC_GAMMA28:
+        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_GAMMA28;
+        break;
+      case AVCOL_TRC_SMPTE240M:
+        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_SMPTE240M;
+        break;
+      case AVCOL_TRC_LINEAR:
+        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_GAMMA10;
+        break;
+      case AVCOL_TRC_LOG:
+        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_LOG100;
+        break;
+      case AVCOL_TRC_LOG_SQRT:
+        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_LOG316;
+        break;
+      case AVCOL_TRC_BT2020_12:
+        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_BT2020_12;
+        break;
+      default:
+        break;
+    }
+  }
+
+  if (!gst_structure_has_field (in_s, "colorimetry")
+      || in_info->colorimetry.matrix == GST_VIDEO_COLOR_MATRIX_UNKNOWN) {
+    switch (context->colorspace) {
+      case AVCOL_SPC_RGB:
+        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_RGB;
+        break;
+      case AVCOL_SPC_BT709:
+        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_BT709;
+        break;
+      case AVCOL_SPC_FCC:
+        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_FCC;
+        break;
+      case AVCOL_SPC_BT470BG:
+      case AVCOL_SPC_SMPTE170M:
+        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_BT601;
+        break;
+      case AVCOL_SPC_SMPTE240M:
+        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_SMPTE240M;
+        break;
+      case AVCOL_SPC_BT2020_NCL:
+        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_BT2020;
+        break;
+      default:
+        break;
+    }
+  }
+
+  if (!gst_structure_has_field (in_s, "colorimetry")
+      || in_info->colorimetry.range == GST_VIDEO_COLOR_RANGE_UNKNOWN) {
+    if (context->color_range == AVCOL_RANGE_JPEG) {
+      out_info->colorimetry.range = GST_VIDEO_COLOR_RANGE_0_255;
+    } else {
+      out_info->colorimetry.range = GST_VIDEO_COLOR_RANGE_16_235;
+    }
   }
 
   /* try to find a good framerate */
@@ -1083,13 +1272,8 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
   /* calculate and update par now */
   gst_ffmpegviddec_update_par (ffmpegdec, in_info, out_info);
 
-  /* Copy stereo/multiview info from upstream if set */
-  if (GST_VIDEO_INFO_MULTIVIEW_MODE (in_info) != GST_VIDEO_MULTIVIEW_MODE_NONE) {
-    GST_VIDEO_INFO_MULTIVIEW_MODE (out_info) =
-        GST_VIDEO_INFO_MULTIVIEW_MODE (in_info);
-    GST_VIDEO_INFO_MULTIVIEW_FLAGS (out_info) =
-        GST_VIDEO_INFO_MULTIVIEW_FLAGS (in_info);
-  }
+  GST_VIDEO_INFO_MULTIVIEW_MODE (out_info) = ffmpegdec->cur_multiview_mode;
+  GST_VIDEO_INFO_MULTIVIEW_FLAGS (out_info) = ffmpegdec->cur_multiview_flags;
 
   if (!gst_video_decoder_negotiate (GST_VIDEO_DECODER (ffmpegdec)))
     goto negotiate_failed;
@@ -1120,6 +1304,9 @@ negotiate_failed:
     ffmpegdec->pic_height = 0;
     ffmpegdec->pic_par_n = 0;
     ffmpegdec->pic_par_d = 0;
+    ffmpegdec->pic_interlaced = 0;
+    ffmpegdec->pic_field_order = 0;
+    ffmpegdec->pic_field_order_changed = FALSE;
     ffmpegdec->ctx_ticks = 0;
     ffmpegdec->ctx_time_n = 0;
     ffmpegdec->ctx_time_d = 0;
@@ -1362,6 +1549,31 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
   gst_buffer_replace (&out_frame->output_buffer, out_dframe->buffer);
   gst_buffer_replace (&out_dframe->buffer, NULL);
 
+  /* Extract auxilliary info not stored in the main AVframe */
+  {
+    GstVideoInfo *in_info = &ffmpegdec->input_state->info;
+    /* Take multiview mode from upstream if present */
+    ffmpegdec->picture_multiview_mode = GST_VIDEO_INFO_MULTIVIEW_MODE (in_info);
+    ffmpegdec->picture_multiview_flags =
+        GST_VIDEO_INFO_MULTIVIEW_FLAGS (in_info);
+
+    /* Otherwise, see if there's info in the frame */
+    if (ffmpegdec->picture_multiview_mode == GST_VIDEO_MULTIVIEW_MODE_NONE) {
+      AVFrameSideData *side_data =
+          av_frame_get_side_data (ffmpegdec->picture, AV_FRAME_DATA_STEREO3D);
+      if (side_data) {
+        AVStereo3D *stereo = (AVStereo3D *) side_data->data;
+        ffmpegdec->picture_multiview_mode = stereo_av_to_gst (stereo->type);
+        if (stereo->flags & AV_STEREO3D_FLAG_INVERT) {
+          ffmpegdec->picture_multiview_flags =
+              GST_VIDEO_MULTIVIEW_FLAGS_RIGHT_VIEW_FIRST;
+        } else {
+          ffmpegdec->picture_multiview_flags = GST_VIDEO_MULTIVIEW_FLAGS_NONE;
+        }
+      }
+    }
+  }
+
   GST_DEBUG_OBJECT (ffmpegdec,
       "pts %" G_GUINT64_FORMAT " duration %" G_GUINT64_FORMAT,
       out_frame->pts, out_frame->duration);
@@ -1737,6 +1949,9 @@ gst_ffmpegviddec_stop (GstVideoDecoder * decoder)
   ffmpegdec->pic_height = 0;
   ffmpegdec->pic_par_n = 0;
   ffmpegdec->pic_par_d = 0;
+  ffmpegdec->pic_interlaced = 0;
+  ffmpegdec->pic_field_order = 0;
+  ffmpegdec->pic_field_order_changed = FALSE;
   ffmpegdec->ctx_ticks = 0;
   ffmpegdec->ctx_time_n = 0;
   ffmpegdec->ctx_time_d = 0;
@@ -2085,6 +2300,20 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
       goto next;
     }
 
+    if (g_str_has_suffix (in_plugin->name, "_qsv")) {
+      GST_DEBUG
+          ("Ignoring qsv decoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      goto next;
+    }
+
+    if (g_str_has_suffix (in_plugin->name, "_cuvid")) {
+      GST_DEBUG
+          ("Ignoring CUVID decoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      goto next;
+    }
+
     GST_DEBUG ("Trying plugin %s [%s]", in_plugin->name, in_plugin->long_name);
 
     /* no codecs for which we're GUARANTEED to have better alternatives */
